# %%
import pandas as pd
import Levenshtein
import numpy as np
import matplotlib.pyplot as plt
from tqdm import tqdm


# %%
data_path = '../../data_import/exports/data_mapping_mdm.csv'
df = pd.read_csv(data_path, skipinitialspace=True)
mdm_list = sorted(list((set(df['pattern']))))


# %%
df['thing_property'] = df['thing'] + df['property']

# %%
def compute_norm_leven(string1, string2):
    max_distance = max(len(string1), len(string2))
    leven_distance = Levenshtein.distance(string1, string2)
    norm_leven = leven_distance / max_distance
    return norm_leven

# %%
n = len(df)
distance_array = np.zeros((n), dtype=float)

desc_array = df['tag_description']
thing_property_array = df['thing_property']

# %%
# compute normalized levenshtein distance
for i in tqdm(range(n)):
    string1 = desc_array[i]
    string2 = thing_property_array[i]
    distance_array[i] = compute_norm_leven(string1, string2)

# %%
distance_array


# %%
plt.figure(figsize=(8, 6)) 
plt.hist(distance_array, bins=30, color='steelblue', edgecolor='black', alpha=0.7)
plt.xlabel("Normalized Levenshtein Distance")
plt.ylabel("Count")
plt.tight_layout()
plt.savefig("histogram.png", dpi=300)
#
# %%
# summary statistics of computed levenshtein distance
def summary_stats(arr):
    return {
        "Mean": np.mean(arr),
        "Median": np.median(arr),
        "Standard Deviation": np.std(arr),
        "Variance": np.var(arr),
        "Min": np.min(arr),
        "Max": np.max(arr),
        "Range": np.ptp(arr),
        "25th Percentile": np.percentile(arr, 25),
        "75th Percentile": np.percentile(arr, 75),
        "Sum": np.sum(arr),
    }

stats = summary_stats(distance_array)

for key, value in stats.items():
    print(f"{key}: {value}")

# %%
